/**
 * This module uses the popups infrastructure to build popups that can show a drupal page.
 * Any link can trigger this effect by setting the attribute popups=1.
 * This module can also be used to build other types of popups.
 * Here is an example:
 * Drupal.popups.prototype.open_unsaved = function( a ) {
 *   body = Drupal.t("There are unsaved changes on this page.");
 *   buttons = {
 *    'popups_save': { title: Drupal.t('Save Changes'), func: function(){$('#edit-submit').click()} },
 *    'popups_submit': { title: Drupal.t('Discard Changes and Continue'), func: function(){window.location = a.href} },
 *    'popups_cancel': { title: Drupal.t('Cancel'), func: this.close }
 *  };
 *  this.open( "Warning - Please Confirm", body, buttons );
 *  return false;
 * };
 */

Drupal.behaviors.popups = function(context) {
  var popups = new Drupal.popups();
  // Add the popup-link-in-dialog behavor (this might not want to be a reattachable behavior, maybe just on page load)
  $(Drupal.settings.popups.popupSelector, context).click( function(e){ 
    if (!e.shiftKey) { // only do popup if shift is not pressed.
      return popups.open_path(this, 800);
    } 
  });
  // Expiremental features.
  $('a[popups]', context).click( function(){ return popups.open_path(this, 800); } );
  $('a[swap]', context).click( function(){ return Drupal.popups.swap(this); } );
};

Drupal.popups = function() {
  this.default_width = 480;
};

/**
 * Generic dialog builder
 */
Drupal.popups.prototype.open = function( title, body, buttons, width ) {
  if( !width ) {
    width = this.default_width;
  }

  var $overlay = Drupal.popups.get_overlay();
  if( !$overlay ) {
    $overlay = Drupal.popups.add_overlay();
  }
    
  // center on the screen, add in offsets if the window has been scrolled
  var overlay_width = $overlay.width();
  var overlay_height = $overlay.height();
  var left = (overlay_width / 2) - (width / 2) + Drupal.popups.f_scrollLeft();
  var top = 0; // we will reposition after adding to page
  // Start with dialog off the side. Making it invisible causes flash.
  var popup = Drupal.theme('popupsDialog', -9999, top, width, title, body, buttons);

  $overlay.before( popup ); // popups is added, but still hidden.  

  // Adding button functions
  if (buttons) {
	  for (var id in buttons) {
	    if (buttons[id]) { // to make jslint happy.
  	    var func = buttons[id].func;
	      $('#'+id).click( func );
	    }
	  }
	}
  $('#popups-close').click( Drupal.popups.close );
  
  // Get popups's height on the page, and center vertically before showing.
  var popups_height = $('#popups').height(); // Causes flash if not visible!
  if( popups_height < overlay_height ) {
    top = ( overlay_height / 2 ) - ( popups_height / 2 ) + Drupal.popups.f_scrollTop();
  }
  else { // popups is too big to fit on screen
    top = 20  + Drupal.popups.f_scrollTop();
  }
//  console.log( "Setting top to: " + top );
  $('#popups').css('top', top).css('left', left);
  
  Drupal.popups.remove_loading(); // Remove the loading img.
   
  return false;
};

/**
 * Use Ajax to open the link in a popup window.
 */
Drupal.popups.prototype.open_path = function( a, width, height ) {
  var url = a.href;
  var popup = this;
  // let the user know something is happening
  $('body').css("cursor", "wait");
  var $overlay = Drupal.popups.add_overlay(); 
  Drupal.popups.add_loading();

  // Load the link's path into the popups, instead of going to the new page.
  var popup_url = Drupal.settings.basePath + "popups/raw?path=" + url.replace('?', '&');
  $.get(popup_url, function(data) {
     var $data = $(data);
     var title = $data.find('#raw-title').html();
     var content = $data.find('#raw-content').html();
             
     popup.open( title, content, null, 600 );
     // If the popup contains a form, capture submits.
     $('form', '#popups-body').ajaxForm({ 
       beforeSubmit: Drupal.popups.beforeSubmit,
       success: Drupal.popups.success
     });
     $('body').css("cursor", "auto"); // Return the cursor to normal state.
  });        
  return false;         
};

Drupal.popups.beforeSubmit = function(form_data, $form, options) {
  Drupal.popups.remove_popup(); // Remove just the dialog, but not the overlay.
  Drupal.popups.add_loading();
  // Send the original page back to Drupal as a flag to return the form results unthemed. 
  var url = options.url + "&original=" + location.href;
  options.url = url;
};

Drupal.popups.success = function (response, status) {
  var $data = $(response);
  var message;
  if ($data.size() > 1) { // bad html response
//    title = 'Error: Bad response.';
    var $response = $('<div></div>');
    $response.html( response );
    message = 'Messages: ' + $('.messages', $response).text();
    Drupal.popups.message('Error: No content returned.', message);
  }
  else {
	  var selector = Drupal.settings.popups.contentSelector;
	  
	  // scan for messages.
	  message = $data.find('#raw-messages').html();
	  if (message) {
	    Drupal.popups.message(message);
//	    var popup = new Drupal.popups();
//	    var buttons = {
//	      'popups_ok': { title: Drupal.t('OK'), func: Drupal.popups.close }
//	    };
//	    popup.open( message, '', buttons );
	  }
	  
	  var content = $data.find('#raw-content').html();
	  var $c = $(selector).html(content); // Inject the new content into the page.
	  Drupal.attachBehaviors($c);
  }
  
  Drupal.popups.remove_loading(); // $('#popups-loading').remove();
  if (!message) {
    Drupal.popups.remove_overlay(); //$('#popups-overlay').remove();
  }
}; 

Drupal.popups.message = function (message, body) {
  var popup = new Drupal.popups();
  var buttons = {
    'popups_ok': { title: Drupal.t('OK'), func: Drupal.popups.close }
  };
  popup.open( message, body, buttons );
};

/**
 * Theme Functions ******************************************************************
 */

Drupal.theme.prototype.popupsLoading = function (left, top) {
  var loading = '<div id="popups-loading">';
  loading += '<div style="left:' + left +'px; top:' + top +'px;">';
  loading += '<img src="'+ Drupal.settings.basePath +'/sites/all/modules/popups/ajax-loader.gif" />';
  loading += '</div></div>';
  return loading;
};

Drupal.theme.prototype.popupsOverlay = function () {
  return '<div id="popups-overlay"></div>';
};

Drupal.theme.prototype.popupsDialog = function(left, top, width, title, body, buttons) {
  var popups = '<div  id="popups" style="left:'+ left +'px; top:'+ top +'px;';
  popups += ' width:'+ width +'px;';
  popups += '" >';
  popups += '<div id="popups-title">'+
            '  <div id="popups-close"><a>close [X]</a></div>'+
            '  <div class="title">' + title +'</div>'+
            '  <div class="clear"></div>'+
            '</div>';
  if (body) {
    popups += '<div id="popups-body">' + body +'</div>';
  }
  popups += '  <div id="popups-buttons">';

  for ( var id in buttons) {
    if (buttons[id]) {
      var button = buttons[id];
      popups += '<input type="button" value="'+ button.title +'" id="'+ id +'" />';
    }
  }
  
  popups += '  </div>'; // end buttons
  popups +=  '</div>'; // close popups
  return popups;
};

/**
 * Appearence function (overlay, loading graphic, remove popup) ****************************
 */
 
Drupal.popups.add_overlay = function() {
  var $overlay = $( Drupal.theme('popupsOverlay') );

  $overlay.css( 'opacity', '0.4' ); // for ie (?)
  $('body').prepend( $overlay );
//  this.add_loading();
  return $overlay;
};

Drupal.popups.get_overlay = function() {
  return $('#popups-overlay');
};

Drupal.popups.remove_overlay = function() {
  $('#popups-overlay').remove();
};

Drupal.popups.add_loading = function() {
  var $overlay = Drupal.popups.get_overlay();
  var wait_image_size = 100;
  var left = ( $overlay.width() / 2 ) - ( wait_image_size / 2 );
  var top = ( $overlay.height() / 2 ) - ( wait_image_size / 2 );
  var $loading = $( Drupal.theme('popupsLoading', left, top) );
  $('body').prepend($loading);
};

Drupal.popups.remove_loading = function() {
  $('#popups-loading').remove();
};

Drupal.popups.remove_popup = function() {
  $('#popups').remove();
};

/**
 *  Remove everything.
 */
Drupal.popups.close = function() {
  Drupal.popups.remove_popup();
  Drupal.popups.remove_loading();
  Drupal.popups.remove_overlay();
};

/**
 * Expiremental Functions ***********************************************
 */
 
 Drupal.popups.swap = function( clicked ) {
  // let the user know something is happening
  $('body').css("cursor", "wait");
  var url = clicked.href;

  // loaded the link's path into the popups, instead going to new page  
  $.get(Drupal.settings.basePath + "popups/raw?path="+url, function(data) {
    var wrapper = $(clicked).attr('swap');
    $(wrapper).wrapAll('<div id="temp_for_swap">');
    $('#temp_for_swap').replaceWith(data);
    $('body').css("cursor", "auto");
  });        
  return false;         
};

// Do an Ahah refresh of the content on the current page.
Drupal.popups.reload = function() {
  // Let the user know something is happening.
  $('body').css("cursor", "wait");
  var selector = Drupal.settings.popups.contentSelector;
  $(selector).css( 'opacity', '0.4' );
  
  // Swap out the contents of the page with fresh generated contents of the same page.
  $.get(Drupal.settings.basePath + "popups/raw?path="+location.href, function(data) {
    var content = $(data).filter('#raw-content').html();   
    var $c = $(selector).html(content);
    Drupal.attachBehaviors($c);
    // Turn off the notifications.
    $('body').css("cursor", "auto");
  });        
  return false;         
};
 

/**
 * Utility functions taken from http://www.softcomplex.com/docs/get_window_size_and_scrollbar_position.html
 */

//function f_scrollLeft() {
Drupal.popups.f_scrollLeft = function() {
  return Drupal.popups.f_filterResults (
    window.pageXOffset ? window.pageXOffset : 0,
    document.documentElement ? document.documentElement.scrollLeft : 0,
    document.body ? document.body.scrollLeft : 0 );
};
//function f_scrollTop() {
Drupal.popups.f_scrollTop = function() {
  return Drupal.popups.f_filterResults (
    window.pageYOffset ? window.pageYOffset : 0,
    document.documentElement ? document.documentElement.scrollTop : 0,
    document.body ? document.body.scrollTop : 0 );
};
//function f_filterResults(n_win, n_docel, n_body) {
Drupal.popups.f_filterResults = function(n_win, n_docel, n_body) {
  var n_result = n_win ? n_win : 0;
  if (n_docel && (!n_result || (n_result > n_docel))) {
    n_result = n_docel;
  }
  return n_body && (!n_result || (n_result > n_body)) ? n_body : n_result;
};


// temp
/**
 * This would be improved if I could convince Drupal to return just the content, instead of the 
 *  entire page, but filtering works.
 */
Drupal.popups.success_old = function (response, status) {
  // Wrap the page in a div or jQuery crashes.
  var $response = $('<div></div>');
  $response.html( response );
  var selector = Drupal.settings.popups.contentSelector;
  var serverSelector = Drupal.settings.popups.servedContentSelector; // servedContentSelector
  var $content;
  if (serverSelector) {
    $content = $(serverSelector, $response);
  }
  else {
    $content = $(selector, $response);
  }
  
  // scan for messages.
  var message = $('.messages', $response).text();
  if (message) {
    var popup = new Drupal.popups();
    var buttons = {
      'popups_ok': { title: Drupal.t('OK'), func: Drupal.popups.close }
    };
    popup.open( message, '', buttons );
  }
  var $c = $(selector).html($content);
  Drupal.attachBehaviors($c);
  
  Drupal.popups.remove_loading(); // $('#popups-loading').remove();
  if (!message) {
    Drupal.popups.remove_overlay(); //$('#popups-overlay').remove();
  }
}; 